%testScriptBVE.m
%
% Test some blood vel. est. algorithms    
%
% Erik Gudmundson,  14 Jan. 2011
%

clear 
disp(['Date and time: ' datestr(now)])

%whatTest = 1; %Tests flowshot ones(32,1)
%whatTest = 2; %Tests flowshot [1 1 0]
whatTest = 3; %Tests flowshot [1 1 1 1 0 0 0 0 0 0 0 0 0]


SNR = [-20 -15 -10 -5 0 5];
MC = 100;
saveData = 0
doPlot = 1

P = 500; %nbr of freq points in the spectrum
%Max nbr of iterations for BIAA
iterMax = 15;

%Make sure we have reset the random variables
% rand('state',sum(100*clock)); 
% randn('state',sum(100*clock));
%rand('state',0); 
%randn('state',0);
randn('state',SNR(1)*MC); %Unique but repeatable
rand('state',SNR(1)*MC);


startTimeTot = cputime;

f_s = 20e6; %Sampling freq of the system
f_c = 5e6; %Center freq. of the transducer
w_c = 2*pi*f_c;
phi = w_c/f_s;

c = 1540; %Speed of sound
f_prf = 15e3; %Pulse repetition freq.
v_z = .2; %Generic axial velocity
psi = -2*w_c/(c*f_prf)*v_z;
velTrue = v_z;

K = 33; %Nbr of fast time samples
if whatTest == 1
  disp(['Doing test 1, Ns=32, with MC=' num2str(MC) '.'])
  Ns = 32;
  flowShots = ones(1,Ns);
  nbrOfBlocks = 1; %Nbr of flowshot blocks
  algNames = {'BIAA','BSLIM','Welch','B-Capon','BAPES'};
  saveStr = ['mseVsSnr_MC' num2str(MC) '_allSNR_Ns32'];
  disp(['Save string: ' saveStr])
  %For the filterbank methods:
  no_filt = Ns-2;
elseif whatTest == 2
  flowShots = [ 1 1 0 ];
  disp(['Doing test 2, [' num2str(flowShots) '], with MC=' num2str(MC) '.'])
  nbrOfBlocks = 12; %Nbr of flowshot blocks
  algNames = {'BIAA','BSLIM','BPG-Capon','BPG-APES'};
  saveStr = ['mseBveVsSnr_MC' num2str(MC) '_allSNR_110'];
  disp(['Save string: ' saveStr])
  %For the filterbank methods:
  NsTil = 2;
  NcTil = 2;
  nbrOfEmptySamples = length(find(flowShots==0));
elseif whatTest == 3
  flowShots = [ 1 1 1 1 0 0 0 0 0 0 0 0 0 ];
  disp(['Doing test 3, [' num2str(flowShots) '], with MC=' num2str(MC) '.'])
  nbrOfBlocks = 10; %Nbr of flowshot blocks
  algNames = {'BIAA','BSLIM','BPG-Capon','BPG-APES'};
  saveStr = ['mseBveVsSnr_MC' num2str(MC) '_allSNR_1111000000000'];
  disp(['Save string: ' saveStr])
  %For the filterbank methods:
  NsTil = 3;
  NcTil = 5;
  nbrOfEmptySamples = length(find(flowShots==0));
else
  error('Script not implemented for this test.')
end
disp(['SNR: [' num2str(SNR) ']'])
whatAlg = 2; %For BIAA and BSLIM
q = 1; %For BSLIM
flowShotsAll = repmat(flowShots,1,nbrOfBlocks)
NVec = find(flowShotsAll);
NAll = length(flowShotsAll);
NVecAll = 0:NAll-1;


%freqVec =  2*pi*( (0:P-1)/P );
freqVecShifted = ((0:P-1)- P/2)/P;
velVec = 2*pi*freqVecShifted*(c*f_prf)/(-2*w_c);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Generate noise-free data
YNoNoise = zeros(K,NAll);
z = exp(j*psi*NVecAll);
for k = 1:K
  YNoNoise(k,:) = exp(j*phi*(k-1))*z;
end
%Compute signal power
sigPower = ( YNoNoise(:)'*YNoNoise(:) )/length(YNoNoise(:));
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Noise variance
snrLen = length(SNR);
sigma2 = sigPower * 10.^(-SNR/10);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Estimation part
nbrOfAlgs = length(algNames);

ampEst = zeros(P,MC,snrLen,nbrOfAlgs);
velEst = zeros(MC,snrLen,nbrOfAlgs);
for k0 = 1:snrLen
  disp(['SNR: ' num2str(SNR(k0)) 'dB'])
  for mc = 1:MC
    if ~rem(mc,10)
      disp(['mc = ' num2str(mc)])
    end
    
    %Create noise vector 
    %(for simplicity, noise will be added also to empty samples)
    noise = sqrt(sigma2(k0)/2)*(randn(K,NAll) + i*randn(K,NAll));
    
    %Add noise to data
    Y = YNoNoise + noise;
    
    %First, we compute using BIAA...
    ampEst(:,mc,k0,1) = fftshift(BIAA( Y, P, phi, NVec, iterMax, whatAlg));
    %...and BSLIM...
    ampEst(:,mc,k0,2) = fftshift(BSLIM( Y, P, phi, NVec, iterMax, whatAlg, q));
    
    %...then the APES and Capon versions
    if whatTest == 1
      [ampEst(:,mc,k0,3),tmp,tmp,ampEst(:,mc,k0,4),...
        tmp,ampEst(:,mc,k0,5),tmp] = ...
        spec_est_3(Y,no_filt,P,phi);
    elseif (whatTest == 2 || whatTest == 3)
      [ampEst(:,mc,k0,3), tmp, ampEst(:,mc,k0,4), tmp] = ...
        pgFiltBankMethods(Y,nbrOfEmptySamples, nbrOfBlocks, NcTil, NsTil, P, phi);
    else
      error('Not yet implemented!')
    end
   
    %Find the peak location and store it
    for ii = 1:nbrOfAlgs
      [tmp ind] = max(ampEst(:,mc,k0,ii));
      velEst(mc,k0,ii) = velVec(ind);
    end
 
  end %of MC
end %of SNR

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Compute MSE
mseVel = zeros(nbrOfAlgs,snrLen);
for ii = 1:nbrOfAlgs;
  mseVel(ii,:) = mean( (velEst(:,:,ii) - velTrue).^2, 1);
end

timeTaken_min = (cputime - startTimeTot)/60
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if doPlot
  algsToTest = 1:nbrOfAlgs;
  
  myLine = {'-x','--o',':*','-.+',...
    '-s','--d',':v','-.^',...
    '-<','-->',':p','-.h'};
  
  figure(); 
  title(['Blood Velocity Estimation, for ' ...
    num2str(MC) 'MC, test ' num2str(whatTest)]); 
  xlabel('SNR'); ylabel('MSE')
  hold all
  count = 0;
  for ii = algsToTest
    count = count + 1;
    plot(SNR,10*log10(mseVel(ii,:)),myLine{count})
    myLegend{count} = algNames{ii};
  end
  legend(myLegend,'location','best')
  grid on
  hold off
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if saveData
  clear ampEst count
  clear startTime startTimeTot
  clear noise Y noisePower
  clear k k0 mc tmp ind ii
  clear saveData doPlot
  save (saveStr) 
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
